package unit1;

public class ListArray implements IListInt {

	private int[] elems; //array to store the elements of the list
	private int maxElems; //maximum number of elements in the list.
	private int nextInserted;// index where it should be stored the next element
	
	/**
	 * Create a list empty. The array has to be initialized in order to be able of storing 
	 * a total of max elements
	 * max is the maximum number of elements that can be stored in this list.
	 * We need this parameter because our implementation uses an array
	 * @param max
	 */
	public ListArray(int max) {
		maxElems=max;
		elems=new int[maxElems];
		nextInserted=0;
	}
	
	@Override
	public void addFirst(int newElem) {
		System.out.println("adding " + newElem + " at the beginning of the list");
		if (nextInserted==maxElems) {
			System.out.println("Sorry, the list is full!!!");
			return;
		}
		if (nextInserted!=0) {
			//we have to move one position all of the elements
			for (int i=nextInserted;i>0;i--) {
				elems[i]=elems[--i];
			}
		}
		elems[0]=newElem;
		nextInserted++;
	}

	@Override
	public void addLast(int newElem) {
		System.out.println("adding " + newElem + " at the end of the list");

		if (nextInserted==maxElems) {
			System.out.println("Sorry, the list is full!!!");
			return;
		}
		elems[nextInserted]=newElem;
		nextInserted++;

	}

	@Override
	public void removeFirst() {
		System.out.println("removing the first elem");

		if (nextInserted==0) {
			System.out.println("Sorry, the list is already empty!!!");
			return;
		}
		for (int i=1;i<nextInserted;i++) {
			elems[i-1]=elems[i];
		}
		nextInserted--;
	}

	@Override
	public void removeLast() {
		System.out.println("removing the last elem");

		if (nextInserted==0) {
			System.out.println("Sorry, the list is already empty!!!");
			return;
		}
		nextInserted--;

	}

	@Override
	public void insertAt(int index, int newElem) {
		
		System.out.println("adding " + newElem + " at the position " + index);

		if (nextInserted==maxElems) {
			System.out.println("Sorry, the list is full!!!");
			return;
		}
		if (index<0 ||index>=maxElems||index>nextInserted) {
			System.out.println("Sorry, " + index + " is not an index correct");
			return;
		}
		for (int i=index;i<nextInserted;i++) {
			elems[++i]=elems[i];
		}
		elems[index]=newElem;
		nextInserted++;
		
		
	}

	@Override
	public boolean isEmpty() {
		return nextInserted==0;
	}

	@Override
	public boolean contains(int elem) {
		boolean found=false;
		for (int i=0;i<nextInserted && !found;i++) {
			if (elem==elems[i]) found=true;
		}
		return found;
	}

	@Override
	public int getSize() {
		return nextInserted;
	}

	@Override
	public int getIndexOf(int elem) {
		int index=-1;
		for (int i=0;i<nextInserted && (index==-1);i++) {
			if (elem==elems[i]) {
				index=i;
			}
		}
		return index;
	}

	@Override
	public int getFirst() {
		if (nextInserted==0) {
			System.out.println("Sorry, the list is  empty!!!");
			return -1;
		}
		return elems[0];
	}

	@Override
	public int getLast() {
		if (nextInserted==0) {
			System.out.println("Sorry, the list is  empty!!!");
			return -1;
		}
		return elems[--nextInserted];
	}

	@Override
	public int getAt(int index) {
		if (index<0 ||index>=maxElems||index>nextInserted) {
			System.out.println("Sorry, " + index + " is not an index correct");
			return -1;
		}
		return elems[index];
		
	}
	
	@Override
	public void removeAt(int index) {
		System.out.println("removing elem in the position " + index);

		if (nextInserted==0) {
			System.out.println("Sorry, the list is already empty!!!");
			return;
		}
		if (index<0 ||index>=maxElems||index>=nextInserted) {
			System.out.println("Sorry, " + index + " is not an index correct");
			return;
		}
		for (int i=index;i<nextInserted;i++) {
			elems[i]=elems[i+1];
		}
		nextInserted--;

	}

	@Override
	public void removeAll(int elem) {
		System.out.println("removing all elements with value " + elem);

		// TODO Auto-generated method stub
		while (contains(elem)) {
			int pos=getIndexOf(elem);
			removeAt(pos);
		}

	}

	public void show() {
		for (int i=0;i<nextInserted;i++) {
			System.out.print(getAt(i)+",");
		}
		System.out.println();
	}

	
	public static void main(String args[]) {
		ListArray list=new ListArray(25);
		list.show();
		list.addFirst(3);
		list.show();
		list.addFirst(5);
		list.show();
		list.addLast(6);
		list.show();
		list.removeFirst();
		list.show();
		list.removeLast();
		list.show();
		list.removeLast();
		list.show();
		for(int i=0;i<=12;i=i+2) {
			list.insertAt(i, i);
			list.insertAt(i+1, i);
			list.show();
		}
		list.addLast(0);
		list.show();
		list.removeAt(6);
		list.show();
		list.removeAt(0);
		list.show();
		list.removeAt(13);
		list.show();
		list.removeAt(list.getSize());
		list.show();

		for (int i=0;i<list.getSize();i++) {
			System.out.println(list.getAt(i));
		}
		
		for (int i=0;i<25;i++) {
			System.out.println("Index of " + i + " = " + list.getIndexOf(i));
		}
	}
}
